@@ -253,6 +253,10 @@ class Queue:
             return False
         return lock_acquired
 
+    def release_maintenance_lock(self):
+        """Deletes the maintenance lock after registries have been cleaned"""
+        self.connection.delete(self.registry_cleaning_key)
+
     def empty(self):
         """Removes all messages on the queue.
         This is currently being done using a Lua script,
@@ -818,6 +822,7 @@ class Queue:
                 "retry": job_data.retry,
                 "on_success": job_data.on_success,
                 "on_failure": job_data.on_failure,
+                "on_stopped": job_data.on_stopped,
             }
 
         # Enqueue jobs without dependencies
@@ -893,7 +898,7 @@ class Queue:
             kwargs (*kwargs): function kargs
         """
         if not isinstance(f, str) and f.__module__ == '__main__':
-            raise ValueError('Functions from the __main__ module cannot be processed ' 'by workers')
+            raise ValueError('Functions from the __main__ module cannot be processed by workers')
 
         # Detect explicit invocations, i.e. of the form:
         #     q.enqueue(foo, args=(1, 2), kwargs={'a': 1}, job_timeout=30)
@@ -1206,6 +1211,7 @@ class Queue:
                         pipeline=pipe,
                         exclude_job_id=exclude_job_id,
                     )
+                    and dependent_job.get_status(refresh=False) != JobStatus.CANCELED
                 ]
 
                 pipe.multi()
