@@ -60,7 +60,7 @@ class BaseDeathPenalty:
 
 class UnixSignalDeathPenalty(BaseDeathPenalty):
     def handle_death_penalty(self, signum, frame):
-        raise self._exception('Task exceeded maximum timeout value ' '({0} seconds)'.format(self._timeout))
+        raise self._exception('Task exceeded maximum timeout value ({0} seconds)'.format(self._timeout))
 
     def setup_death_penalty(self):
         """Sets up an alarm signal and a signal handler that raises
