@@ -1,15 +1,16 @@
-from typing import Optional, TYPE_CHECKING, Any, Set
+from typing import TYPE_CHECKING, Optional, Set
 
 if TYPE_CHECKING:
     from redis import Redis
     from redis.client import Pipeline
-    from .worker import Worker
-    from .queue import Queue
 
-from .utils import as_text
+    from .queue import Queue
+    from .worker import Worker
 
 from rq.utils import split_list
 
+from .utils import as_text
+
 WORKERS_BY_QUEUE_KEY = 'rq:workers:%s'
 REDIS_WORKER_KEYS = 'rq:workers'
 MAX_KEYS = 1000
@@ -86,7 +87,6 @@ def clean_worker_registry(queue: 'Queue'):
     keys = list(get_keys(queue))
 
     with queue.connection.pipeline() as pipeline:
-
         for key in keys:
             pipeline.exists(key)
         results = pipeline.execute()
