@@ -4,6 +4,7 @@ import shutil
 import signal
 import subprocess
 import sys
+import threading
 import time
 import zlib
 from datetime import datetime, timedelta
@@ -268,7 +269,7 @@ class TestWorker(RQTestCase):
         w.perform_job(job, queue)
 
         # An exception should be logged here at ERROR level
-        self.assertIn("Traceback", mock_logger_error.call_args[0][0])
+        self.assertIn("Traceback", mock_logger_error.call_args[0][3])
 
     def test_heartbeat(self):
         """Heartbeat saves last_heartbeat"""
@@ -594,6 +595,91 @@ class TestWorker(RQTestCase):
         # Should not have created evidence of execution
         self.assertEqual(os.path.exists(SENTINEL_FILE), False)
 
+    def test_cancel_running_parent_job(self):
+        """Cancel a running parent job and verify that
+        dependent jobs are not started."""
+
+        def cancel_parent_job(job):
+            while job.is_queued:
+                time.sleep(1)
+
+            job.cancel()
+            return
+
+        q = Queue(
+            "low",
+        )
+        parent_job = q.enqueue(long_running_job, 5)
+
+        job = q.enqueue(say_hello, depends_on=parent_job)
+        job2 = q.enqueue(say_hello, depends_on=job)
+        status_thread = threading.Thread(target=cancel_parent_job, args=(parent_job,))
+        status_thread.start()
+
+        w = Worker([q])
+        w.work(
+            burst=True,
+        )
+        status_thread.join()
+
+        self.assertNotEqual(parent_job.result, None)
+        self.assertEqual(job.get_status(), JobStatus.DEFERRED)
+        self.assertEqual(job.result, None)
+        self.assertEqual(job2.get_status(), JobStatus.DEFERRED)
+        self.assertEqual(job2.result, None)
+        self.assertEqual(q.count, 0)
+
+    def test_cancel_dependent_job(self):
+        """Cancel job and verify that when the parent job is finished,
+        the dependent job is not started."""
+
+        q = Queue(
+            "low",
+        )
+        parent_job = q.enqueue(long_running_job, 5, job_id="parent_job")
+        job = q.enqueue(say_hello, depends_on=parent_job, job_id="job1")
+        job2 = q.enqueue(say_hello, depends_on=job, job_id="job2")
+        job.cancel()
+
+        w = Worker([q])
+        w.work(
+            burst=True,
+        )
+        self.assertTrue(job.is_canceled)
+        self.assertNotEqual(parent_job.result, None)
+        self.assertEqual(job.get_status(), JobStatus.CANCELED)
+        self.assertEqual(job.result, None)
+        self.assertEqual(job2.result, None)
+        self.assertEqual(job2.get_status(), JobStatus.DEFERRED)
+        self.assertEqual(q.count, 0)
+
+    def test_cancel_job_enqueue_dependent(self):
+        """Cancel a job in a chain and enqueue the dependent jobs."""
+
+        q = Queue(
+            "low",
+        )
+        parent_job = q.enqueue(long_running_job, 5, job_id="parent_job")
+        job = q.enqueue(say_hello, depends_on=parent_job, job_id="job1")
+        job2 = q.enqueue(say_hello, depends_on=job, job_id="job2")
+        job3 = q.enqueue(say_hello, depends_on=job2, job_id="job3")
+
+        job.cancel(enqueue_dependents=True)
+
+        w = Worker([q])
+        w.work(
+            burst=True,
+        )
+        self.assertTrue(job.is_canceled)
+        self.assertNotEqual(parent_job.result, None)
+        self.assertEqual(job.get_status(), JobStatus.CANCELED)
+        self.assertEqual(job.result, None)
+        self.assertNotEqual(job2.result, None)
+        self.assertEqual(job2.get_status(), JobStatus.FINISHED)
+        self.assertEqual(job3.get_status(), JobStatus.FINISHED)
+
+        self.assertEqual(q.count, 0)
+
     @slow
     def test_max_idle_time(self):
         q = Queue()
@@ -943,14 +1029,8 @@ class TestWorker(RQTestCase):
         self.assertEqual(worker.last_cleaned_at, None)
         worker.clean_registries()
         self.assertNotEqual(worker.last_cleaned_at, None)
-        self.assertEqual(self.testconn.zcard(foo_registry.key), 0)
-        self.assertEqual(self.testconn.zcard(bar_registry.key), 0)
-
-        # worker.clean_registries() only runs once every 15 minutes
-        # If we add another key, calling clean_registries() should do nothing
-        self.testconn.zadd(bar_registry.key, {'bar': 1})
-        worker.clean_registries()
-        self.assertEqual(self.testconn.zcard(bar_registry.key), 1)
+        self.assertEqual(len(foo_registry), 0)
+        self.assertEqual(len(bar_registry), 0)
 
     def test_should_run_maintenance_tasks(self):
         """Workers should run maintenance tasks on startup and every hour."""
@@ -973,13 +1053,13 @@ class TestWorker(RQTestCase):
 
     def test_worker_calls_clean_registries(self):
         """Worker calls clean_registries when run."""
-        queue = Queue(connection=self.testconn)
-        registry = StartedJobRegistry(connection=self.testconn)
+        queue = Queue(connection=self.connection)
+        registry = StartedJobRegistry(connection=self.connection)
         self.testconn.zadd(registry.key, {'foo': 1})
 
-        worker = Worker(queue, connection=self.testconn)
+        worker = Worker(queue, connection=self.connection)
         worker.work(burst=True)
-        self.assertEqual(self.testconn.zcard(registry.key), 0)
+        self.assertEqual(len(registry), 0)
 
     def test_job_dependency_race_condition(self):
         """Dependencies added while the job gets finished shouldn't get lost."""
@@ -1165,7 +1245,7 @@ class TestWorker(RQTestCase):
 
     def test_request_force_stop_ignores_consecutive_signals(self):
         """Ignore signals sent within 1 second of the last signal"""
-        queue = Queue(connection=self.testconn)
+        queue = Queue(connection=self.connection)
         worker = Worker([queue])
         worker._horse_pid = 1
         worker._shutdown_requested_date = utcnow()
